;**************************************************************************
;* FileName	: NLCD.INC 
;* Date 	: 09.10.2002
;* File version : v1.04
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: code for LTSCC7021A LCD Module
;*		  ==================================
;*		  NLCD_init
;*		  NLCD_don		(*)
;*		  NLCD_doff		(*)
;*		  NLCD_clear		(*)
;*		  NLCD_clearrest	(*)
;*		  NLCD_gotox
;*		  NLCD_symbol		(*)
;*		  NLCD_printc
;*		  NLCD_printcs		(*)
;*
;*                (remark: * - conditional compiled)
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* #define		NLCD_CS0	PORTB,0		; out
;* #define		NLCD_CS1	PORTB,1		; out
;* #define		NLCD_CLK	PORTB,2		; out
;* #define		NLCD_SI		PORTB,3		; in
;*
;*	include "..\modules\wait.mac"
;*	include "..\modules\wait.inc"
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define		NLCD_RESET	PORTB,4		; LCD -RESET pin
;* #define		NLCD_ALPHA			; alphanumeric charset
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  01/03/2001 	Initial Release
;* 1.01   12/05/2001	new function: NLCD_clearrest
;* 1.02   25/09/2001	new function: NLCD_printcs
;* 1.03   15/06/2002	minor changes
;* 1.04   09/10/2002    reformatting source  
;*
;**************************************************************************




;***** VARIABLES **********************************************************

	cblock
		NLCD_data
		NLCD_cycle
		NLCD_decoded
		NLCD_pointer
	endc

#ifdef NLCD_PRINTCS_INC
	cblock
		NLCD_addrlow
		NLCD_addrhigh
	endc
#endif

;***** NLCD_init **********************************************************
;* Initialization of the LCD module
;* Input : -             		; 
;* Output: - 				; 
;* Clock : ca. 6,5 mS (+50 mS time of LCD Reset)
;**************************************************************************
NLCD_init	bank1				; set TRIS bists
		ERRORLEVEL -302
		bcf	NLCD_CS0
		bcf	NLCD_CS1
		bcf	NLCD_CLK
		bcf	NLCD_SI
#ifdef NLCD_RESET
		bcf	NLCD_RESET
#endif
		bank0
		ERRORLEVEL +302
		bsf	NLCD_CS0
		bsf	NLCD_CS1
		bcf	NLCD_CLK
		
#ifdef NLCD_RESET
		bcf	NLCD_RESET
		WAIT	50 Millisec,0
		bsf	NLCD_RESET
#endif
		clrf	NLCD_pointer
		bcf	NLCD_CS0		; init 2st chip
		call	NLCD_init_chip
		bsf	NLCD_CS0
		bcf	NLCD_CS1		; init 1st chip
		call	NLCD_init_chip
		bsf	NLCD_CS1
		return



;***** NLCD_init_chip *****************************************************
;* Initialization a chip PD7225
;* Input : -				; 
;* Output: - 				; 
;* Clock : ca. 3,2 mS
;**************************************************************************
NLCD_init_chip	movlw	NLCD_mode_val		; mode_set
		call	NLCD_send
		movlw	NLCD_cmd_wosd		; without segment decoder
		call	NLCD_send
		movlw	NLCD_cmd_clear		; clear data memory
		call	NLCD_send
		movlw	NLCD_cmd_unsync		; unsync. transfer
		call	NLCD_send
		movlw	NLCD_cmd_d_on		; display on
		call	NLCD_send
		return


;***** NLCD_don ***********************************************************
;* Switch on the display
;* Input : -				; 
;* Output: - 				; 
;* Clock : ca. 1.3 mS
;**************************************************************************
	ifdef	NLCD_DON_INC
NLCD_don	bcf	NLCD_CS1
		movlw	NLCD_cmd_d_on
		call	NLCD_send
		bsf	NLCD_CS1
		bcf	NLCD_CS0
		movlw	NLCD_cmd_d_on
		call	NLCD_send
		bsf	NLCD_CS0
		return
	endif



;***** NLCD_doff **********************************************************
;* Switch off the display
;* Input : -				; 
;* Output: - 				; 
;* Clock : ca. 1.3 mS
;**************************************************************************
	ifdef	NLCD_DOFF_INC
NLCD_doff	bcf	NLCD_CS1
		movlw	NLCD_cmd_d_off
		call	NLCD_send
		bsf	NLCD_CS1
		bcf	NLCD_CS0
		movlw	NLCD_cmd_d_off
		call	NLCD_send
		bsf	NLCD_CS0
		return
	endif
	


;***** NLCD_symbol ********************************************************
;* Display symbols
;* Input : symbols -> W			; 
;* Output: - 				; 
;* Clock : ca. 2.8 mS  
;**************************************************************************
	ifdef	NLCD_SYMBOL_INC
NLCD_symbol	movwf	NLCD_decoded
		bcf	NLCD_CS0
		movlw	24
		iorlw	NLCD_cmd_setp
		call	NLCD_send
		decf	NLCD_pointer, f
		goto	NLCD_send_d1
	endif



;***** NLCD_clear *********************************************************
;* Clear display
;* Input : -				; 
;* Output: - 				; 
;* Clock : ca. 1.3 mS
;**************************************************************************
	ifdef	NLCD_CLEAR_INC
NLCD_clear	bcf	NLCD_CS1
		movlw	NLCD_cmd_clear	
		call	NLCD_send
		bsf	NLCD_CS1
		bcf	NLCD_CS0
		movlw	NLCD_cmd_clear	
		call	NLCD_send
		bsf	NLCD_CS0
		clrf	NLCD_pointer
		return
	endif



;***** NLCD_clearrest *****************************************************
;* Clear rest of line
;* Input : -				; 
;* Output: - 				; 
;* Clock : ?
;**************************************************************************
	ifdef	NLCD_CREST_INC
NLCD_clearrest	movf NLCD_pointer,w
		skipnz
		return

		movlw	' '
		call	NLCD_printc
		goto	NLCD_clearrest
	endif



;***** NLCD_gotox *********************************************************
;* Set print position on the LCD
;* Input : W <- position  (unsigned)	; if W>17 then NLCD_pointer = 0
;*					; else NLCD_pointer = W
;* Output: - 				; 
;* Clock : 6  
;**************************************************************************
NLCD_gotox	movwf	NLCD_pointer
		sublw	18-1
		skipc
		clrf	NLCD_pointer
		return




;***** NLCD_printc ********************************************************
;* print an ASCII character to the LCD
;*  MSB bit of W -> DP (DotPoint)
;*  legal chars if NLCD_ALPHA not definied : - . / 0-9
;*  legal chars if NLCD_ALPHA definied     : - . / 0-9 @ A-Z [ \ ] ^ _
;*  others are replaced with space.
;**************************************************************************
;* Input : W <- char      		; ASCII code of the char
;* Output: - 				; -
;* Clock : ca. 2.8 mS  
;**************************************************************************
NLCD_printc	movwf	NLCD_decoded		; store char

		movf	NLCD_pointer,w
		sublw	9			; set CARRY
		movf	NLCD_pointer,w
		jnc	NLCD_printc_1
				
		bcf	NLCD_CS1
		goto	NLCD_printc_2

NLCD_printc_1	bcf	NLCD_CS0
		addlw	-30

NLCD_printc_2	addwf	NLCD_pointer,w
		addwf	NLCD_pointer,w
		iorlw	NLCD_cmd_setp
		call	NLCD_send

		movf	NLCD_decoded, w

; ----- NLCD send decoded begin -------------------------------------------

		clrf	NLCD_decoded
		movwf	NLCD_data
		btfsc	NLCD_data, 7		; set DP
		bsf	NLCD_decoded, 0
		bcf	NLCD_data, 7

#ifdef NLCD_ALPHA
		btfsc	NLCD_data, 6
		goto	NLCD_decode_1
#endif
		
		; ----- numeric data --------------------------------------

		movf	NLCD_data,w		; skip if data > '9'		
		sublw	'9'
		skipc
		goto	NLCD_send_d1

		movlw	'-'			; skip if data < '-'	
		subwf	NLCD_data,w
		skipc
		goto	NLCD_send_d1

		movwf	NLCD_data
		call	NLCD_num_table
		iorwf	NLCD_decoded, f
		

#ifdef NLCD_ALPHA
		goto	NLCD_send_d1

NLCD_decode_1
		; ----- alpha data  ---------------------------------------

		bcf	NLCD_data, 6
		bcf	NLCD_data, 5

		call	NLCD_alpha_table
		iorwf	NLCD_decoded, f
#endif

NLCD_send_d1	movlw	NLCD_cmd_setmem
		movwf	NLCD_data
		btfsc	NLCD_decoded, 6
		bsf	NLCD_data, 0
		btfsc	NLCD_decoded, 5
		bsf	NLCD_data, 1
		btfsc	NLCD_decoded, 0
		bsf	NLCD_data, 2
		movf	NLCD_data,w
		call	NLCD_send

		movlw	NLCD_cmd_setmem
		movwf	NLCD_data
		btfsc	NLCD_decoded, 7
		bsf	NLCD_data, 0
		btfsc	NLCD_decoded, 1
		bsf	NLCD_data, 1
		btfsc	NLCD_decoded, 4
		bsf	NLCD_data, 2
		movf	NLCD_data,w
		call	NLCD_send

		movlw	NLCD_cmd_setmem
		movwf	NLCD_data
		btfsc	NLCD_decoded, 2
		bsf	NLCD_data, 0
		btfsc	NLCD_decoded, 3
		bsf	NLCD_data, 1
		movf	NLCD_data,w
		call	NLCD_send

		incf	NLCD_pointer, f

		movlw	18
		xorwf	NLCD_pointer, w
		skipnz
		clrf	NLCD_pointer

; ----- NLCD send decoded end ---------------------------------------------

		bsf	NLCD_CS1
		bsf	NLCD_CS0
		WAIT	160 Microsec, 0		; High level CSx pulse width
		return



;***** NLCD_num_table *****************************************************
;* decode numeric segments
;* Input : NLCD_data           		; ASCII code of the char
;* Output: segs -> W			; LCD segmens of the char :
;* 					;      abcdefgD
;*					;   MSB^      ^LSB 
;* Clock : 10
;**************************************************************************
NLCD_num_table	movlw	HIGH NLCD_numt1
		movwf	PCLATH

		movf	NLCD_data,w
		addlw	LOW NLCD_numt1		; set CARRY
		skipnc
		incf	PCLATH,f
		movwf	PCL

NLCD_numt1	retlw	00000010b		; '-'
		retlw	00000001b		; '.'
		retlw	01001010b		; '/'
		retlw	11111100b		; '0'
		retlw	01100000b		; '1'
		retlw	11011010b		; '2'
		retlw	11110010b		; '3'
		retlw	01100110b		; '4'
		retlw	10110110b		; '5'
		retlw	10111110b		; '6'
		retlw	11100100b		; '7'
		retlw	11111110b		; '8'
		retlw	11110110b		; '9'



;***** NLCD_alpha_table ***************************************************
;* decode alpha segments
;* Input : NLCD_data           		; ASCII code of the char
;* Output: segs -> W			; LCD segmens of the char :
;* 					;      abcdefgD
;*					;   MSB^      ^LSB 
;* Clock : 10
;**************************************************************************
#ifdef NLCD_ALPHA
NLCD_alpha_table
		movlw	HIGH NLCD_alphat1
		movwf	PCLATH

		movf	NLCD_data,w
		addlw	LOW NLCD_alphat1	; set CARRY
		skipnc
		incf	PCLATH,f
		movwf	PCL

NLCD_alphat1	retlw	11111000b		; '@'
		retlw	11101110b		; 'A'
		retlw	00111110b		; 'B'
		retlw	10011100b		; 'C'
		retlw	01111010b		; 'D'
		retlw	10011110b		; 'E'
		retlw	10001110b		; 'F'
		retlw	10111100b		; 'G'
		retlw	01101110b		; 'H'
		retlw	01100000b		; 'I'
		retlw	01111000b		; 'J'
		retlw	01001110b		; 'K'
		retlw	00011100b		; 'L'
		retlw	10101010b		; 'M'
		retlw	11101100b		; 'N'
		retlw	11111100b		; 'O'
		retlw	11001110b		; 'P'
		retlw	11100110b		; 'Q'
		retlw	10001100b		; 'R'
		retlw	10110110b		; 'S'
		retlw	11100000b		; 'T'
		retlw	01111100b		; 'U'
		retlw	01001100b		; 'V'
		retlw	01010110b		; 'W'
		retlw	01001011b		; 'X'
		retlw	01110110b		; 'Y'
		retlw	11011010b		; 'Z'
		retlw	10011100b		; '['
		retlw	00100110b		; '\'
		retlw	11110000b		; ']'
		retlw	10000000b		; '^'
		retlw	00010000b		; '_'
#endif



;***** NLCD_send **********************************************************
; Send command to the LCD module
;* Input : W <- command			;
;* Output: - 				;
;* Clock : ca. 0.64 mS
;**************************************************************************
NLCD_send	movwf	NLCD_data

		WAIT	500 Microsec, 0	; Busy low level time, unsync.

		movlw	8			; 8 bit
		movwf	NLCD_cycle


NLCD_s1		bcf	NLCD_CLK
		
		bcf	STATUS,C		; 0 -> CARRY    
		rlf	NLCD_data,f
		skipc		
		bcf	NLCD_SI			; clear ...
		skipnc
		bsf	NLCD_SI			; or set si

		WAIT	2 Microsec, 0

		bsf	NLCD_CLK

		WAIT	2 Microsec, 0

		decfsz	NLCD_cycle,f	
		goto	NLCD_s1

		return



;***** NLCD_printcs *******************************************************
;* Print a constant string to the LCD
;* Input : w <- addr_high		; high address of cstring
;* Output: -				;
;* Clock : ?		
;**************************************************************************
#ifdef NLCD_PRINTCS_INC
NLCD_printcs	movwf	NLCD_addrhigh

NLCD_print1	call	NLCD_print_getc
		testw
		skipnz
		return

		call	NLCD_printc

		incf	NLCD_addrlow,f
		skipnz
		incf	NLCD_addrhigh,f
		goto	NLCD_print1

NLCD_print_getc	movf	NLCD_addrhigh,w
		movwf	PCLATH
		movf	NLCD_addrlow,w
		movwf	PCL		; computed goto!

#endif



;**************************************************************************




